@@ -312,6 +312,7 @@ get_graphics_data_glx_renderer ()
     None
   };
   int nconfigs;
+  int major, minor;
   Window window;
   GLXFBConfig *config;
   GLXWindow glxwin;
@@ -324,6 +325,7 @@ get_graphics_data_glx_renderer ()
 
   display = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
 
+  glXQueryVersion (display, &major, &minor);
   config = glXChooseFBConfig (display, DefaultScreen (display),
                               attributes, &nconfigs);
   if (config == NULL) {
@@ -823,7 +825,7 @@ set_virtualization_label (CcInfoPanel  *self,
   display_name = NULL;
   for (i = 0; i < G_N_ELEMENTS (virt_tech); i++)
     {
-      if (g_str_equal (virt_tech[i].id, virt) == 0)
+      if (g_str_equal (virt_tech[i].id, virt))
         {
           display_name = _(virt_tech[i].display);
           break;
@@ -1838,9 +1840,25 @@ got_pk_proxy_cb (GObject *source_object,
     }
 }
 
+static gboolean
+does_prepared_update_exist (void)
+{
+  return g_file_test ("/var/lib/PackageKit/prepared-update", G_FILE_TEST_EXISTS);
+}
+
 static void
 info_panel_setup_updates (CcInfoPanel *self)
 {
+  if (does_gnome_software_exist ())
+    {
+      if (does_prepared_update_exist ())
+        self->priv->updates_state = UPDATES_AVAILABLE;
+      else
+        self->priv->updates_state = UPDATES_NOT_AVAILABLE;
+      refresh_update_button (self);
+      return;
+    }
+
   self->priv->updates_state = CHECKING_UPDATES;
   refresh_update_button (self);
 
