@@ -87,6 +87,7 @@ static GtkWidget *custom_shortcut_command_entry = NULL;
 static GHashTable *kb_system_sections = NULL;
 static GHashTable *kb_apps_sections = NULL;
 static GHashTable *kb_user_sections = NULL;
+static gpointer wm_changed_id = NULL;
 
 static void
 free_key_array (GPtrArray *keys)
@@ -1843,8 +1844,8 @@ void
 keyboard_shortcuts_init (CcPanel *panel, GtkBuilder *builder)
 {
   g_object_set_data (G_OBJECT (panel), "builder", builder);
-  wm_common_register_window_manager_change ((GFunc) on_window_manager_change,
-                                            panel);
+  wm_changed_id = wm_common_register_window_manager_change ((GFunc) on_window_manager_change,
+                                                            panel);
   pictures_regex = g_regex_new ("\\$PICTURES", 0, 0, NULL);
   setup_dialog (panel, builder);
   reload_sections (panel);
@@ -1928,5 +1929,11 @@ keyboard_shortcuts_dispose (CcPanel *panel)
 
   g_clear_pointer (&custom_shortcut_dialog, gtk_widget_destroy);
 
+  if (wm_changed_id)
+    {
+      wm_common_unregister_window_manager_change (wm_changed_id);
+      wm_changed_id = NULL;
+    }
+
   cc_keyboard_option_clear_all ();
 }
