@@ -58,6 +58,8 @@ static void populate_ap_list (NetDeviceWifi *device_wifi);
 struct _NetDeviceWifiPrivate
 {
         GtkBuilder              *builder;
+        GtkWidget               *details_dialog;
+        GtkWidget               *hotspot_dialog;
         gboolean                 updating_device;
         gchar                   *selected_ssid_title;
         gchar                   *selected_connection_id;
@@ -87,7 +89,6 @@ device_wifi_proxy_add_to_notebook (NetObject *object,
                                     GtkSizeGroup *heading_size_group)
 {
         GtkWidget *widget;
-        GtkWindow *window;
         NetDeviceWifi *device_wifi = NET_DEVICE_WIFI (object);
 
         /* add widgets to size group */
@@ -95,15 +96,9 @@ device_wifi_proxy_add_to_notebook (NetObject *object,
                                                      "heading_ipv4"));
         gtk_size_group_add_widget (heading_size_group, widget);
 
-        /* reparent */
-        window = GTK_WINDOW (gtk_builder_get_object (device_wifi->priv->builder,
-                                                     "window_tmp"));
         widget = GTK_WIDGET (gtk_builder_get_object (device_wifi->priv->builder,
                                                      "notebook_view"));
-        g_object_ref (widget);
-        gtk_container_remove (GTK_CONTAINER (window), widget);
         gtk_notebook_append_page (notebook, widget, NULL);
-        g_object_unref (widget);
 
         return widget;
 }
@@ -305,6 +300,9 @@ device_is_hotspot (NetDeviceWifi *device_wifi)
         NMDevice *device;
 
         device = net_device_get_nm_device (NET_DEVICE (device_wifi));
+        if (nm_device_get_active_connection (device) == NULL)
+                return FALSE;
+
         c = find_connection_for_device (device_wifi, device);
         if (c == NULL)
                 return FALSE;
@@ -510,8 +508,7 @@ nm_device_wifi_refresh_ui (NetDeviceWifi *device_wifi)
 
         nm_device = net_device_get_nm_device (NET_DEVICE (device_wifi));
 
-        dialog = GTK_WIDGET (gtk_builder_get_object (device_wifi->priv->builder,
-                                                     "details_dialog"));
+        dialog = device_wifi->priv->details_dialog;
 
         ap = g_object_get_data (G_OBJECT (dialog), "ap");
         connection = g_object_get_data (G_OBJECT (dialog), "connection");
@@ -635,6 +632,20 @@ connect_to_hidden_network (NetDeviceWifi *device_wifi)
         cc_network_panel_connect_to_hidden_network (toplevel, client, remote_settings);
 }
 
+static void
+connection_add_or_activate_cb (NMActiveConnection *connection,
+                               GError *error,
+                               NetDeviceWifi *device_wifi)
+{
+        if (connection == NULL) {
+                /* failed to activate */
+                g_debug ("Failed to add and/or activate connection '%d': %s",
+                         error->code,
+                         error->message);
+                nm_device_wifi_refresh_ui (device_wifi);
+        }
+}
+
 static void
 connection_add_activate_cb (NMClient *client,
                             NMActiveConnection *connection,
@@ -642,15 +653,7 @@ connection_add_activate_cb (NMClient *client,
                             GError *error,
                             gpointer user_data)
 {
-        NetDeviceWifi *device_wifi = user_data;
-
-        if (connection == NULL) {
-                /* failed to activate */
-                g_debug ("Failed to add and activate connection '%d': %s",
-                         error->code,
-                         error->message);
-                nm_device_wifi_refresh_ui (device_wifi);
-        }
+        connection_add_or_activate_cb (connection, error, user_data);
 }
 
 static void
@@ -659,15 +662,7 @@ connection_activate_cb (NMClient *client,
                         GError *error,
                         gpointer user_data)
 {
-        NetDeviceWifi *device_wifi = user_data;
-
-        if (connection == NULL) {
-                /* failed to activate */
-                g_debug ("Failed to activate connection '%d': %s",
-                         error->code,
-                         error->message);
-                nm_device_wifi_refresh_ui (device_wifi);
-        }
+        connection_add_or_activate_cb (connection, error, user_data);
 }
 
 static gboolean
@@ -763,19 +758,20 @@ wireless_try_to_connect (NetDeviceWifi *device_wifi,
                                                        connection_add_activate_cb, device_wifi);
         } else {
                 CcNetworkPanel *panel;
-                GPtrArray *array;
+                GVariantBuilder *builder;
+                GVariant *parameters;
 
                 g_debug ("no existing connection found for %s, creating", ssid_target);
-                array = g_ptr_array_new ();
-                g_ptr_array_add (array, "connect-8021x-wifi");
-                g_ptr_array_add (array, (gpointer) nm_object_get_path (NM_OBJECT (device)));
-                g_ptr_array_add (array, (gpointer) ap_object_path);
-                g_ptr_array_add (array, NULL);
+                builder = g_variant_builder_new (G_VARIANT_TYPE ("av"));
+                g_variant_builder_add (builder, "v", g_variant_new_string ("connect-8021x-wifi"));
+                g_variant_builder_add (builder, "v", g_variant_new_string (nm_object_get_path (NM_OBJECT (device))));
+                g_variant_builder_add (builder, "v", g_variant_new_string (ap_object_path));
+                parameters = g_variant_new ("av", builder);
 
                 panel = net_object_get_panel (NET_OBJECT (device_wifi));
-                g_object_set (G_OBJECT (panel), "argv", array->pdata, NULL);
+                g_object_set (G_OBJECT (panel), "parameters", parameters, NULL);
 
-                g_ptr_array_free (array, FALSE);
+                g_variant_builder_unref (builder);
         }
 out:
         return;
@@ -1111,7 +1107,7 @@ start_hotspot (GtkButton *button, NetDeviceWifi *device_wifi)
 
         window = gtk_widget_get_toplevel (GTK_WIDGET (button));
 
-        dialog = GTK_WIDGET (gtk_builder_get_object (device_wifi->priv->builder, "hotspot-dialog"));
+        dialog = device_wifi->priv->hotspot_dialog;
         gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
 
         str = g_string_new (_("If you have a connection to the Internet other than wireless, you can set up a wireless hotspot to share the connection with others."));
@@ -1295,6 +1291,8 @@ net_device_wifi_finalize (GObject *object)
         NetDeviceWifi *device_wifi = NET_DEVICE_WIFI (object);
         NetDeviceWifiPrivate *priv = device_wifi->priv;
 
+        g_clear_pointer (&priv->details_dialog, gtk_widget_destroy);
+        g_clear_pointer (&priv->hotspot_dialog, gtk_widget_destroy);
         g_object_unref (priv->builder);
         g_free (priv->selected_ssid_title);
         g_free (priv->selected_connection_id);
@@ -1374,7 +1372,7 @@ really_forget (GtkDialog *dialog, gint response, gpointer data)
 
         gtk_widget_destroy (GTK_WIDGET (dialog));
 
-        if (response == GTK_RESPONSE_CANCEL)
+        if (response != GTK_RESPONSE_OK)
                 return;
 
         device_wifi = NET_DEVICE_WIFI (g_object_get_data (G_OBJECT (forget), "net"));
@@ -1419,7 +1417,7 @@ check_toggled (GtkToggleButton *check, GtkWidget *forget)
         GtkWidget *row;
         GList *rows;
 
-        row = gtk_widget_get_parent (GTK_WIDGET (check));
+        row = gtk_widget_get_ancestor (GTK_WIDGET (check), GTK_TYPE_LIST_BOX_ROW);
         active = gtk_toggle_button_get_active (check);
         rows = g_object_steal_data (G_OBJECT (forget), "rows");
 
@@ -1433,6 +1431,18 @@ check_toggled (GtkToggleButton *check, GtkWidget *forget)
         gtk_widget_set_sensitive (forget, rows != NULL);
 }
 
+static void
+update_forget (GtkWidget *forget,
+               gpointer   row)
+{
+        GList *rows;
+
+        rows = g_object_steal_data (G_OBJECT (forget), "rows");
+        rows = g_list_remove (rows, row);
+        g_object_set_data_full (G_OBJECT (forget), "rows", rows, (GDestroyNotify)g_list_free);
+        gtk_widget_set_sensitive (forget, rows != NULL);
+}
+
 static void
 make_row (GtkSizeGroup   *rows,
           GtkSizeGroup   *icons,
@@ -1513,6 +1523,8 @@ make_row (GtkSizeGroup   *rows,
                 gtk_widget_set_halign (widget, GTK_ALIGN_CENTER);
                 gtk_widget_set_valign (widget, GTK_ALIGN_CENTER);
                 gtk_box_pack_start (GTK_BOX (row_box), widget, FALSE, FALSE, 0);
+                g_signal_connect_object (row, "destroy",
+                                         G_CALLBACK (update_forget), forget, G_CONNECT_SWAPPED);
         }
         if (check_out)
                 *check_out = widget;
@@ -1549,7 +1561,7 @@ make_row (GtkSizeGroup   *rows,
                 atk_object_set_name (gtk_widget_get_accessible (widget), _("Options…"));
                 gtk_box_pack_start (GTK_BOX (row_box), widget, FALSE, FALSE, 0);
                 gtk_size_group_add_widget (spinner_button_group, widget);
-                g_object_set_data (G_OBJECT (row_box), "edit", widget);
+                g_object_set_data (G_OBJECT (row), "edit", widget);
         }
         if (edit_out)
                 *edit_out = widget;
@@ -1564,7 +1576,7 @@ make_row (GtkSizeGroup   *rows,
         gtk_widget_set_valign (widget, GTK_ALIGN_CENTER);
         gtk_box_pack_start (GTK_BOX (row_box), widget, FALSE, FALSE, 0);
         gtk_size_group_add_widget (spinner_button_group, widget);
-        g_object_set_data (G_OBJECT (row_box), "spinner", widget);
+        g_object_set_data (G_OBJECT (row), "spinner", widget);
 
         box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
         gtk_box_set_homogeneous (GTK_BOX (box), TRUE);
@@ -1597,8 +1609,11 @@ make_row (GtkSizeGroup   *rows,
         gtk_widget_show_all (row);
 
         g_object_set_data (G_OBJECT (row), "ap", ap);
-        g_object_set_data (G_OBJECT (row), "connection", connection);
-
+        if (connection) {
+                g_object_set_data (G_OBJECT (row), "connection", connection);
+                g_signal_connect_object (connection, "removed",
+                                         G_CALLBACK (gtk_widget_destroy), row, G_CONNECT_SWAPPED);
+        }
         g_object_set_data (G_OBJECT (row), "timestamp", GUINT_TO_POINTER (timestamp));
         g_object_set_data (G_OBJECT (row), "active", GUINT_TO_POINTER (active));
         g_object_set_data (G_OBJECT (row), "strength", GUINT_TO_POINTER (strength));
@@ -1886,10 +1901,8 @@ ap_activated (GtkListBox *list, GtkListBoxRow *row, NetDeviceWifi *device_wifi)
         edit = GTK_WIDGET (g_object_get_data (G_OBJECT (row), "edit"));
 
         if (ap != NULL) {
-                gtk_widget_hide (edit);
-                gtk_widget_show (spinner);
-                gtk_spinner_start (GTK_SPINNER (spinner));
                 if (connection != NULL) {
+                        gtk_widget_hide (edit);
                         client = net_object_get_client (NET_OBJECT (device_wifi));
                         nm_device = net_device_get_nm_device (NET_DEVICE (device_wifi));
                         nm_client_activate_connection (client,
@@ -1900,6 +1913,10 @@ ap_activated (GtkListBox *list, GtkListBoxRow *row, NetDeviceWifi *device_wifi)
                         const GByteArray *ssid;
                         gchar *ssid_text;
                         const gchar *object_path;
+
+                        gtk_widget_show (spinner);
+                        gtk_spinner_start (GTK_SPINNER (spinner));
+
                         ssid = nm_access_point_get_ssid (ap);
                         ssid_text = g_markup_escape_text (nm_utils_escape_ssid (ssid->data, ssid->len), -1);
                         object_path = nm_object_get_path (NM_OBJECT (ap));
@@ -1931,6 +1948,14 @@ net_device_wifi_init (NetDeviceWifi *device_wifi)
                 return;
         }
 
+        widget = GTK_WIDGET (gtk_builder_get_object (device_wifi->priv->builder,
+                                                     "details_dialog"));
+        device_wifi->priv->details_dialog = widget;
+
+        widget = GTK_WIDGET (gtk_builder_get_object (device_wifi->priv->builder,
+                                                     "hotspot-dialog"));
+        device_wifi->priv->hotspot_dialog = widget;
+
         /* setup wifi views */
         widget = GTK_WIDGET (gtk_builder_get_object (device_wifi->priv->builder,
                                                      "device_off_switch"));
