@@ -140,22 +140,15 @@ net_virtual_device_add_to_notebook (NetObject *object,
 {
         NetVirtualDevice *virtual_device = NET_VIRTUAL_DEVICE (object);
         GtkWidget *widget;
-        GtkWindow *window;
 
         /* add widgets to size group */
         widget = GTK_WIDGET (gtk_builder_get_object (virtual_device->priv->builder,
                                                      "heading_ipv4"));
         gtk_size_group_add_widget (heading_size_group, widget);
 
-        /* reparent */
-        window = GTK_WINDOW (gtk_builder_get_object (virtual_device->priv->builder,
-                                                     "window_tmp"));
         widget = GTK_WIDGET (gtk_builder_get_object (virtual_device->priv->builder,
                                                      "vbox6"));
-        g_object_ref (widget);
-        gtk_container_remove (GTK_CONTAINER (window), widget);
         gtk_notebook_append_page (notebook, widget, NULL);
-        g_object_unref (widget);
         return widget;
 }
 
