@@ -188,9 +188,11 @@ typedef struct
   PpDevicesList *devices;
   GMainContext  *context;
   gboolean       waiting;
+  gboolean       auth_if_needed;
   GtkWindow     *parent;
   SMBAuthInfo   *auth_info;
   gboolean       hostname_set;
+  gboolean       cancelled;
 } SMBData;
 
 static void
@@ -213,6 +215,7 @@ smb_data_free (SMBData *data)
 
 static void
 auth_cb (PpAuthenticationDialog *auth_dialog,
+         gint                    response_id,
          const gchar            *username,
          const gchar            *password,
          gpointer                user_data)
@@ -233,6 +236,10 @@ auth_cb (PpAuthenticationDialog *auth_dialog,
 
   g_object_unref (auth_dialog);
 
+  if (response_id == GTK_RESPONSE_CANCEL ||
+      response_id == GTK_RESPONSE_DELETE_EVENT)
+    data->cancelled = TRUE;
+
   data->waiting = FALSE;
 }
 
@@ -272,41 +279,44 @@ auth_fn (SMBCCTX    *smb_context,
 
   data = (SMBData *) smbc_getOptionUserData (smb_context);
 
-  data->auth_info = g_new (SMBAuthInfo, 1);
-  data->auth_info->server = g_strdup (server);
-  data->auth_info->share = g_strdup (share);
-  data->auth_info->workgroup = g_strdup (workgroup);
-  data->auth_info->username = g_strdup (username);
-  data->auth_info->password = g_strdup (password);
-
-  data->waiting = TRUE;
-
-  source = g_idle_source_new ();
-  g_source_set_callback (source,
-                         get_auth_info,
-                         data,
-                         NULL);
-  g_source_attach (source, data->context);
-  g_source_unref (source);
-
-  /*
-   * smbclient needs to get authentication data
-   * from this synchronous callback so we are blocking
-   * until we get them
-   */
-  while (data->waiting)
+  if (!data->cancelled)
     {
-      g_usleep (POLL_DELAY);
+      data->auth_info = g_new (SMBAuthInfo, 1);
+      data->auth_info->server = g_strdup (server);
+      data->auth_info->share = g_strdup (share);
+      data->auth_info->workgroup = g_strdup (workgroup);
+      data->auth_info->username = g_strdup (username);
+      data->auth_info->password = g_strdup (password);
+
+      data->waiting = TRUE;
+
+      source = g_idle_source_new ();
+      g_source_set_callback (source,
+                             get_auth_info,
+                             data,
+                             NULL);
+      g_source_attach (source, data->context);
+      g_source_unref (source);
+
+      /*
+       * smbclient needs to get authentication data
+       * from this synchronous callback so we are blocking
+       * until we get them
+       */
+      while (data->waiting)
+        {
+          g_usleep (POLL_DELAY);
+        }
+
+      if (g_strcmp0 (username, data->auth_info->username) != 0)
+        g_strlcpy (username, data->auth_info->username, unmaxlen);
+
+      if (g_strcmp0 (password, data->auth_info->password) != 0)
+        g_strlcpy (password, data->auth_info->password, pwmaxlen);
+
+      smb_auth_info_free (data->auth_info);
+      data->auth_info = NULL;
     }
-
-  if (g_strcmp0 (username, data->auth_info->username) != 0)
-    g_strlcpy (username, data->auth_info->username, unmaxlen);
-
-  if (g_strcmp0 (password, data->auth_info->password) != 0)
-    g_strlcpy (password, data->auth_info->password, pwmaxlen);
-
-  smb_auth_info_free (data->auth_info);
-  data->auth_info = NULL;
 }
 
 static void
@@ -347,15 +357,42 @@ list_dir (SMBCCTX      *smb_context,
       dir = smbclient_opendir (smb_context, dirname);
       if (!dir && errno == EACCES)
         {
-          smbc_setFunctionAuthDataWithContext (smb_context, auth_fn);
-          dir = smbclient_opendir (smb_context, dirname);
-          smbc_setFunctionAuthDataWithContext (smb_context, anonymous_auth_fn);
+          if (data->auth_if_needed)
+            {
+              data->cancelled = FALSE;
+              smbc_setFunctionAuthDataWithContext (smb_context, auth_fn);
+              dir = smbclient_opendir (smb_context, dirname);
+              smbc_setFunctionAuthDataWithContext (smb_context, anonymous_auth_fn);
+
+              if (data->cancelled)
+                {
+                  device = g_new0 (PpPrintDevice, 1);
+                  device->host_name = g_str_has_prefix (dirname, "smb://") ? g_strdup (dirname + 6) : g_strdup (dirname);
+                  device->is_authenticated_server = TRUE;
+
+                  data->devices->devices = g_list_append (data->devices->devices, device);
+
+                  if (dir)
+                    smbclient_closedir (smb_context, dir);
+                  return;
+                }
+            }
+          else
+            {
+              device = g_new0 (PpPrintDevice, 1);
+
+              device->host_name = g_str_has_prefix (dirname, "smb://") ? g_strdup (dirname + 6) : g_strdup (dirname);
+              device->is_authenticated_server = TRUE;
+
+              data->devices->devices = g_list_append (data->devices->devices, device);
+            }
         }
 
       while (dir && (dirent = smbclient_readdir (smb_context, dir)))
         {
           gchar *subdirname = NULL;
           gchar *subpath = NULL;
+          gchar *uri;
 
           if (dirent->smbc_type == SMBC_WORKGROUP)
             {
@@ -373,9 +410,12 @@ list_dir (SMBCCTX      *smb_context,
             {
               device = g_new0 (PpPrintDevice, 1);
 
-              device->device_uri = g_strdup_printf ("%s/%s",
-                                                    dirname,
-                                                    dirent->name);
+              uri = g_strdup_printf ("%s/%s", dirname, dirent->name);
+              device->device_uri = g_uri_escape_string (uri,
+                                                        G_URI_RESERVED_CHARS_GENERIC_DELIMITERS
+                                                        G_URI_RESERVED_CHARS_SUBCOMPONENT_DELIMITERS,
+                                                        FALSE);
+              g_free (uri);
 
               device->device_class = g_strdup ("network");
               device->device_info = g_strdup (dirent->comment);
@@ -463,6 +503,7 @@ _pp_samba_get_devices_thread (GSimpleAsyncResult *res,
 
 void
 pp_samba_get_devices_async (PpSamba             *samba,
+                            gboolean             auth_if_needed,
                             GCancellable        *cancellable,
                             GAsyncReadyCallback  callback,
                             gpointer             user_data)
@@ -477,6 +518,7 @@ pp_samba_get_devices_async (PpSamba             *samba,
   data->context = g_main_context_default ();
   data->hostname_set = priv->hostname != NULL;
   data->parent = g_object_ref (priv->parent);
+  data->auth_if_needed = auth_if_needed;
 
   g_simple_async_result_set_check_cancellable (res, cancellable);
   g_simple_async_result_set_op_res_gpointer (res, data, (GDestroyNotify) smb_data_free);
