@@ -100,7 +100,7 @@ cc_sharing_panel_master_switch_notify (GtkSwitch      *gtkswitch,
       gtk_switch_set_active (GTK_SWITCH (WID ("share-public-folder-on-network-switch")),
                              FALSE);
       gtk_switch_set_active (GTK_SWITCH (WID ("share-media-switch")), FALSE);
-      gtk_switch_set_active (GTK_SWITCH (WID ("share-public-folder-switch")),
+      gtk_switch_set_active (GTK_SWITCH (WID ("save-received-files-to-downloads-switch")),
                              FALSE);
     }
 
@@ -348,7 +348,7 @@ bluetooth_set_accept_files (const GValue       *value,
   if (g_value_get_boolean (value))
     return g_variant_new_string ("bonded");
   else
-    return g_variant_new_string ("always");
+    return g_variant_new_string ("ask");
 }
 
 #ifdef HAVE_BLUETOOTH
@@ -384,6 +384,10 @@ cc_sharing_panel_setup_bluetooth_sharing_dialog (CcSharingPanel *self)
   g_signal_connect_swapped (priv->bluetooth_killswitch, "state-changed",
                             G_CALLBACK (bluetooth_state_changed), self);
 
+  cc_sharing_panel_bind_switch_to_label (self,
+                                         WID ("save-received-files-to-downloads-switch"),
+                                         WID ("bluetooth-sharing-status-label"));
+
   cc_sharing_panel_bind_switch_to_widgets (WID ("save-received-files-to-downloads-switch"),
                                            WID ("receive-files-grid"),
                                            NULL);
@@ -413,7 +417,7 @@ cc_sharing_panel_add_folder (GtkWidget      *button,
   gchar *folder;
   GtkTreeIter iter;
   gboolean valid;
-  gboolean matching;
+  gboolean matching = FALSE;
 
   dialog = gtk_file_chooser_dialog_new (_("Choose a Folder"),
                                         GTK_WINDOW (gtk_widget_get_toplevel (button)),
@@ -429,6 +433,10 @@ cc_sharing_panel_add_folder (GtkWidget      *button,
                                                    "shared-folders-liststore");
 
   folder = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dialog));
+  if (!folder || g_str_equal (folder, ""))
+    goto bail;
+
+  g_debug ("Trying to add %s", folder);
 
   for (valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter);
        valid;
@@ -437,18 +445,21 @@ cc_sharing_panel_add_folder (GtkWidget      *button,
       gchar *string;
 
       gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, 0, &string, -1);
-      matching = (g_strcmp0 (string, folder) != 0);
+      matching = (g_strcmp0 (string, folder) == 0);
       g_free (string);
 
       if (matching)
-        break;
+        {
+          g_debug ("Found a duplicate for %s", folder);
+          break;
+        }
     }
 
-  if (!matching && folder && !g_str_equal (folder, ""))
+  if (!matching)
     gtk_list_store_insert_with_values (store, NULL, -1, 0, folder, -1);
 
+bail:
   g_free (folder);
-
   gtk_widget_destroy (dialog);
 }
 
