@@ -1342,7 +1342,7 @@ enterprise_init (UmAccountDialog *self,
                                                self, NULL);
 
         monitor = g_network_monitor_get_default ();
-        g_signal_connect (monitor, "network-changed", G_CALLBACK (on_network_changed), self);
+        g_signal_connect_object (monitor, "network-changed", G_CALLBACK (on_network_changed), self, 0);
 }
 
 static void
@@ -1581,6 +1581,8 @@ um_account_dialog_dispose (GObject *obj)
                 self->enterprise_domain_timeout_id = 0;
         }
 
+        g_clear_pointer (&self->join_dialog, gtk_widget_destroy);
+
         G_OBJECT_CLASS (um_account_dialog_parent_class)->dispose (obj);
 }
 
