@@ -203,12 +203,16 @@ button_changed_cb (GtkComboBox *combo, gpointer user_data)
 			    BUTTONNUMBER_COLUMN, &mapping_b2,
 			    -1);
 
-	if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (WID ("combo-topbutton")), &iter))
-		return;
+	if (gsd_wacom_stylus_get_num_buttons (priv->stylus) > 1) {
+		if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (WID ("combo-topbutton")), &iter))
+			return;
 
-	gtk_tree_model_get (GTK_TREE_MODEL (liststore), &iter,
-			    BUTTONNUMBER_COLUMN, &mapping_b3,
-			    -1);
+		gtk_tree_model_get (GTK_TREE_MODEL (liststore), &iter,
+				    BUTTONNUMBER_COLUMN, &mapping_b3,
+				    -1);
+	} else {
+		mapping_b3 = 0;
+	}
 
 	map_button (priv->stylus_settings, mapping_b2, mapping_b3);
 }
@@ -297,10 +301,10 @@ cc_wacom_stylus_page_init (CcWacomStylusPage *self)
 
 	priv->builder = gtk_builder_new ();
 
-	gtk_builder_add_objects_from_file (priv->builder,
-					   GNOMECC_UI_DIR "/wacom-stylus-page.ui",
-					   objects,
-					   &error);
+	gtk_builder_add_objects_from_resource (priv->builder,
+                                               "/org/gnome/control-center/wacom/wacom-stylus-page.ui",
+                                               objects,
+                                               &error);
 	if (error != NULL) {
 		g_warning ("Error loading UI file: %s", error->message);
 		g_object_unref (priv->builder);
@@ -339,16 +343,13 @@ set_icon_name (CcWacomStylusPage *page,
 	       const char  *icon_name)
 {
 	CcWacomStylusPagePrivate *priv;
-	char *filename, *path;
+	char *resource;
 
 	priv = page->priv;
 
-	filename = g_strdup_printf ("%s.svg", icon_name);
-	path = g_build_filename (GNOMECC_UI_DIR, filename, NULL);
-	g_free (filename);
-
-	gtk_image_set_from_file (GTK_IMAGE (WID (widget_name)), path);
-	g_free (path);
+	resource = g_strdup_printf ("/org/gnome/control-center/wacom/%s.svg", icon_name);
+	gtk_image_set_from_resource (GTK_IMAGE (WID (widget_name)), resource);
+	g_free (resource);
 }
 
 /* Different types of layout for the stylus config */
